@@ -3,41 +3,42 @@ import os.path
 
 from errors import FileFormatError, VersionError
 import utils
+from json_utils import JsonObject
 
+json_properties = [
+    "product", "product_version", "django_settings_module",
+    # python_path_subdirectory is the directory which should be set as the PYTHONPATH,
+    # relative from the top level directory of the application archive.
+    # If the PYTHONPATH should point to the parent directory of the application
+    # archive, then python_path_subdirectory will be the empty string.
+    "python_path_subdirectory",
+    # version of the SDK
+    "version",
+    "installed_apps",
+    "fixtures", "migration_apps", "components",
+    "post_install_commands",
+    "media_root_subdir", "media_url_path",
+    "static_root_subdir", "static_url_path"
+]
 
-class DjangoConfig(object):
-    def __init__(self, product, product_version, django_settings_module, python_path_subdirectory, version,
-                 installed_apps=[], fixtures=[], migration_apps=[], components=[],
-                 post_install_commands=[]):
-        self.product = product
-        self.product_version = product_version
-        self.django_settings_module = django_settings_module
-        # python_path_subdirectory is the directory which should be set as the PYTHONPATH,
-        # relative from the top level directory of the application archive.
-        # If the PYTHONPATH should point to the parent directory of the application
-        # archive, then python_path_subdirectory will be the empty string.
-        self.python_path_subdirectory = python_path_subdirectory
-        self.installed_apps = installed_apps
-        self.fixtures = fixtures
-        self.version = version # version of the SDK
-        self.migration_apps = migration_apps
-        self.components = components
-        self.post_install_commands = post_install_commands
+required_json_properties = [
+    "product", "product_version", "django_settings_module",
+    "python_path_subdirectory",
+    "version"
+]
+
+class DjangoConfig(JsonObject):
+    def __init__(self, properties):
+        JsonObject.__init__(self, json_properties,
+                            required_properties=required_json_properties,
+                            prop_val_dict=properties)
+        # setup defaults for list properties
+        if not self.installed_apps: self.installed_apps = []
+        if not self.fixtures: self.fixtures = []
+        if not self.migration_apps: self.migration_apps = []
+        if not self.components: self.components = []
+        if not self.post_install_commands: self.post_install_commands = []
 
-    def to_json(self):
-        obj = {
-            "product":self.product,
-            "product_version":self.product_version,
-            "django_settings_module":self.django_settings_module,
-            "python_path_subdirectory":self.python_path_subdirectory,
-            "version":self.version,
-            "installed_apps":self.installed_apps,
-            "fixtures":self.fixtures,
-            "migration_apps":self.migration_apps,
-            "components":self.components,
-            "post_install_commands":self.post_install_commands
-            }
-        return obj
 
     def get_settings_module_file(self):
         """Return the path to the settings file. This is a relative path,
@@ -114,24 +115,8 @@ def django_config_from_json_obj(json_obj, compatible_version):
     if not compatible_versions(version, compatible_version):
         raise VersionError("Application package was built using an obsolete version of the SDK (%s). Please download the latest version and repackage your application." %
                            version)
-    # check for required properties
-    for key in ["product", "product_version", "django_settings_module", "python_path_subdirectory", "installed_apps", "fixtures", "migration_apps"]:
-        if not json_obj.has_key(key):
-            raise FileFormatError("django engage configuration file missing required key %s" % key)
-    # Process optional properties. Properties should only be optional if 1) they have
-    # a sensible default value, and 2) we want to avoid a in-compatible release of
-    # the sdk.
-    for (key, default) in [("components", []), ("post_install_commands", [])]:
-        if not json_obj.has_key(key):
-            json_obj[key] = default
-    return DjangoConfig(json_obj['product'],
-                        json_obj['product_version'],
-                        json_obj['django_settings_module'],
-                        json_obj['python_path_subdirectory'], json_obj['version'],
-                        json_obj['installed_apps'], json_obj['fixtures'],
-                        migration_apps=json_obj['migration_apps'],
-                        components=json_obj['components'],
-                        post_install_commands=json_obj['post_install_commands'])
+    return DjangoConfig(json_obj)
+
 
 def django_config_from_json(json_string, compatible_version):
     try:
@@ -139,3 +124,14 @@ def django_config_from_json(json_string, compatible_version):
     except ValueError, m:
         raise FileFormatError("Unable to parse django engage configuration file: %s" % m)
     return django_config_from_json_obj(json_obj, compatible_version)
+
+def django_config_from_validation_results(django_settings_module, version, results):
+    results_json = results.to_json()
+    results_json["django_settings_module"] = django_settings_module
+    results_json["version"] = version
+    if not results_json["product"]:
+        results_json["product"] = "Custom Django Application"
+    if not results_json["product_version"]:
+        results_json["product_version"] = ""
+    return DjangoConfig(results_json)
+    
