@@ -28,4 +28,14 @@ class VersionError(ValidationError):
     """This error is signaled when the version of the packager used to package the app is
     obsolete.
     """
-    pass
\ No newline at end of file
+    pass
+
+class RequestedPackageError(ValidationError):
+    """This error signals when the user requests installation of a package
+    known to cause problems.
+    """
+    pass
+
+class PipError(ValidationError):
+    """This error is raised when a problem occurs in the execution of pip.
+    """
